/***************************** C SOURCE FILE **********************************
 **
 **  Project:    VPL extension module
 **  Filename:   helloworld.c
 **
 ******************************************************************************
 **
 **  This exports a function which returns a string with 'Hello World'
 **
 *****************************************************************************/

/******************************************************************************
 **
 ** MODULES USED
 **
 *****************************************************************************/
#include <module.h>
#include <string.h>


/******************************************************************************
 **
 ** LOCAL FUNCTIONS
 **
 *****************************************************************************/

static int32 MODDECL HelloWorld(HANDLE* pCPU, __attribute__((unused)) void* pbase)
{
   // Var
   char   *text;
   uint16 strid;

   // Generate output string
   text = "Hello World";

   // Allocate string
   strid = vplStringMake(pCPU, text);
   if (!strid) return 0; // Failed to allocate string

   // Completed
   return strid;
}


/******************************************************************************
 **
 ** FUNCTION TABLE
 **
 *****************************************************************************/
static vplFunctionEntry    ftable[] = { { "HelloWorld", HelloWorld }, { NULL, NULL } };


/******************************************************************************
 **
 ** EXPORTED FUNCTIONS
 **
 *****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// This function is called by the firmware when 'extModuleLoad' is called.
// Any initialization the module might require is performed here, including
// installing functions and function blocks.
//
// Input:
//   None.
//
// Output:
//   None.
//
// Returns:
//   0                  - Success
//   1                  - Failed
//
/////////////////////////////////////////////////////////////////////////////
int MODDECL moduleInit(void)
{
   // Var
   uint16 rc;

   // Initialize module
   rc = vplInstallFunctions("helloworld", ftable);
   if (rc)
   {
      // Install failed
      return 1;
   }

   // Completed
   return 0;
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by the firmware to notify the module about an event.
//
// Input:
//   event              - The event ID.
//   parm               - The event parameter.
//
// Output:
//   None.
//
// Returns:
//   None.
//
/////////////////////////////////////////////////////////////////////////////
void MODDECL moduleNotify(int event, int parm)
{
   switch (event)
   {
      case EVENT_HALT:
         break;

      case EVENT_RESET:
         break;

      case EVENT_SHUTDOWN:
         break;

      case EVENT_POWERFAIL:
         if (parm) {
            // Running on battery
         } else {
            // Running on external power
         }
         break;

      case EVENT_POWERSAVE:
         if (parm) {
            // Suspend system
         } else {
            // Resume system
         }
         break;
   }
}


/******************************************************************************
 **
 **    EOF
 **
 *****************************************************************************/
