/***************************** C SOURCE FILE **********************************
 **
 **  Project:    VPL extension module
 **  Filename:   dt_mod.c
 **
 ******************************************************************************
 **
 **  This exports a function block, which calculates a number from two inputs
 **  and generates a string.
 **
 *****************************************************************************/

/******************************************************************************
 **
 ** MODULES USED
 **
 *****************************************************************************/
#include <module.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

/******************************************************************************
 **
 **    TYPEDEFS AND STRUCTURES
 **
 *****************************************************************************/

/*
;*** FUNCTION stringToUpper ***
;*** var
;stringToUpper                   off=0000 key=530613 sz=02 ty=std ival=0
;err                             off=0002 key=530637 sz=02 ty=std ival=0
;*** var_input
;str                             off=0004 key=486327 sz=02 ty=std ival=0
;*** var_input (access)
;out                             off=0006 key=491856 sz=04 ty=std ival=0
*/
typedef struct {
   /*** var */
   int16       retval;
   int16       error;
   /*** var_input */
   int16       str;
   /*** var_input(access) */
   int16*      out;
} ALIGN_ATTR tdef_strtoupper;

/*
;*** FUNCTION numberToHex ***
;*** var
;numberToHex                     off=0000 key=530673 sz=02 ty=std ival=0
;err                             off=0002 key=530637 sz=02 ty=std ival=0
;*** var_input
;v                               off=0004 key=487001 sz=04 ty=std ival=0
;pad                             off=0008 key=530686 sz=01 ty=std ival=0
;*** var_input (access)
 */
typedef struct {
   /*** var */
   int16       retval;
   int16       error;
   /*** var_input */
   int32       v;
   int8        pad;

} ALIGN_ATTR tdef_numbertohex;


/*
;*** FUNCTION sintTest ***
;*** var
;sintTest                        off=0000 key=530735 sz=01 ty=std ival=0
;err                             off=0001 key=530637 sz=02 ty=std ival=0
;*** var_input
;a                               off=0003 key=503134 sz=01 ty=std ival=0
;*** var_input (access)
;b                               off=0004 key=503138 sz=04 ty=std ival=0
 */
typedef struct {
   /*** var */
   int8        retval;
   int16       error;
   /*** var_input */
   int8        a;
   /*** var_input(access) */
   int8*       b;

} ALIGN_ATTR tdef_sinttest;


/*
;*** FUNCTION intTest ***
;*** var
;intTest                         off=0000 key=530794 sz=02 ty=std ival=0
;err                             off=0002 key=530637 sz=02 ty=std ival=0
;*** var_input
;a                               off=0004 key=503134 sz=02 ty=std ival=0
;*** var_input (access)
;b                               off=0006 key=503138 sz=04 ty=std ival=0
 */
typedef struct {
   /*** var */
   int16       retval;
   int16       error;
   /*** var_input */
   int16       a;
   /*** var_input(access) */
   int16*      b;

} ALIGN_ATTR tdef_inttest;

/*
;*** FUNCTION dintTest ***
;*** var
;dintTest                        off=0000 key=530853 sz=04 ty=std ival=0
;err                             off=0004 key=530637 sz=02 ty=std ival=0
;*** var_input
;a                               off=0006 key=503134 sz=04 ty=std ival=0
;*** var_input (access)
;b                               off=000A key=503138 sz=04 ty=std ival=0
 */
typedef struct {
   /*** var */
   int32       retval;
   int16       error;
   /*** var_input */
   int32       a;
   /*** var_input(access) */
   int32*      b;

} ALIGN_ATTR tdef_dinttest;

/*
;*** FUNCTION floatTest ***
;*** var
;floatTest                       off=0000 key=530912 sz=04 ty=std ival=0.00000
;err                             off=0004 key=530637 sz=02 ty=std ival=0
;padding                         off=0006 key=530941 sz=02 ty=std ival=0
;*** var_input
;a                               off=0008 key=503134 sz=04 ty=std ival=0.00000
;*** var_input (access)
;b                               off=000C key=503138 sz=04 ty=std ival=0.00000
 */
typedef struct {
   /*** var */
   float       retval;
   int16       error;
   int16       padding;// to align float
   /*** var_input */
   float       a;
   /*** var_input(access) */
   float*      b;

} ALIGN_ATTR tdef_floattest;

/*
;*** FUNCTION boolTest ***
;*** var
;boolTest                        off=0000 key=530979 sz=01 ty=std ival=0
;err                             off=0001 key=530637 sz=02 ty=std ival=0
;*** var_input
;a                               off=0003 key=503134 sz=01 ty=std ival=0
;*** var_input (access)
;b                               off=0004 key=503138 sz=04 ty=std ival=0
 */
typedef struct {
   /*** var */
   int8        retval;
   int16       error;
   /*** var_input */
   int8        a;
   /*** var_input(access) */
   int8*       b;

} ALIGN_ATTR tdef_booltest;

/*
;*** STRUCT_BLOCK sb_data ***
;*** var_input
;elements                        off=0000 key=531045 sz=02 ty=std ival=0
;elem                            off=0002 key=531051 sz=14 ty=std ival=0
;Size=0x16 (p-align=0x0)
 */
typedef struct{
   int16 elements;
   int16 elem[10];
} sb_data_t;

/*
;*** FUNCTION sb_test ***
;*** var
;sb_test                         off=0000 key=531067 sz=02 ty=std ival=0
;err                             off=0002 key=530637 sz=02 ty=std ival=0
;*** var_input
;*** var_input (access)
;data                            off=0004 key=487790 sz=04 ty=fub ival=0
 */
typedef struct {
   /*** var */
   int16       retval;
   int16       error;
   /*** var_input */
   //int8        a;
   /*** var_input(access) */
   sb_data_t* data;

} ALIGN_ATTR tdef_sbtest;


/*
;*** FUNCTION get_buf ***
;*** var
;get_buf                         off=0000 key=531110 sz=02 ty=std ival=0
;err                             off=0002 key=530637 sz=02 ty=std ival=0
;*** var_input
;size                            off=0004 key=486247 sz=02 ty=std ival=0
;*** var_input (access)
;buf                             off=0006 key=495944 sz=04 ty=std ival=0
 */
typedef struct {
   /*** var */
   int16       retval;
   int16       error;
   /*** var_input */
   int16       size;
   /*** var_input(access) */
   void** buf;

} ALIGN_ATTR tdef_getbuf;

/*
;*** FUNCTION free_buf ***
;*** var
;free_buf                        off=0000 key=531157 sz=02 ty=std ival=0
;err                             off=0002 key=530637 sz=02 ty=std ival=0
;*** var_input
;buf                             off=0004 key=495944 sz=04 ty=std ival=0
;*** var_input (access)
 */
typedef struct {
   /*** var */
   int16       retval;
   int16       error;
   /*** var_input */
   void*       buf;

} ALIGN_ATTR tdef_freebuf;


/*
;*** FUNCTION_BLOCK fbTest ***
;*** var
;err                             off=0000 key=530637 sz=02 ty=std ival=0
;*** var_input
;in_a                            off=0002 key=531201 sz=02 ty=std ival=0
;in_b                            off=0004 key=531207 sz=01 ty=std ival=0
;*** var_output
;str                             off=0006 key=486327 sz=02 ty=std ival=0
;result                          off=0008 key=531222 sz=02 ty=std ival=0
;Size=0xC (p-align=0x2)
 */
typedef struct {
   /*** var */
   int16       error;
   /*** var_input */
   int16       in_a;
   int8        in_b;
   int8        padding;
   /*** var_output */
   uint16      str;
   int16       result;
} ALIGN_ATTR tdef_fbtest;


/******************************************************************************
 **
 ** LOCAL FUNCTIONS
 **
 *****************************************************************************/


static int32 MODDECL stringToUpper(HANDLE* pCPU, void* pbase)
{
   // Var
   char  text[255];
   const char *str;
   uint16 strid;
   int i;
   tdef_strtoupper *data =(tdef_strtoupper*)pbase;

   // Get input string
   str = vplStringGet(pCPU, data->str);

   text[0] = '\0';
   for(i=0;i< 254 && str[i];i++){
      text[i] = toupper(str[i]);
   }
   // Create output string
   strid = vplStringUpdate(pCPU, read16b(data->out), text);
   // Write output string to access variable
   write16b(data->out, strid);

   // Completed
   return 0;
}


static int32 MODDECL numberToHex(HANDLE* pCPU, void* pbase)
{
   // Var
   char text[255];
   uint16 strid;
   tdef_numbertohex *data =(tdef_numbertohex*)pbase;

   if(data->pad){
      sprintf(text, "%08lx", data->v);
   }else{
      sprintf(text, "%lx", data->v);
   }

   // Send resulting text to debug messages
   //vplDebug(text);

   // Allocate string
   strid = vplStringMake(pCPU, text);

   // Completed
   return strid;
}



static int32 MODDECL sintTest(HANDLE* pCPU __attribute__((unused)) , void* pbase)
{
   // Var
   tdef_sinttest *data =(tdef_sinttest*)pbase;

   // Set access variable
   *data->b = data->a*10;

   // Completed
   return data->a*2;
}


static int32 MODDECL intTest(HANDLE* pCPU __attribute__((unused)) , void* pbase)
{
   // Var
   int16 i;
   tdef_inttest *data =(tdef_inttest*)pbase;

   // Set access variable
   i = data->a*10;
   write16b(data->b, i);

   // Completed
   return data->a*2;
}


static int32 MODDECL dintTest(HANDLE* pCPU __attribute__((unused)) , void* pbase)
{
   // Var
   int32 d;
   tdef_dinttest *data =(tdef_dinttest*)pbase;

   // Set access variable
   d = data->a*10;
   write32b(data->b, d);

   // Completed
   return  data->a*2;
}

static int32 MODDECL floatTest(HANDLE* pCPU __attribute__((unused)) , void* pbase)
{
   // Var
   float res;
   float f;
   tdef_floattest *data =(tdef_floattest*)pbase;

   f = data->a*10.1;
   // Set access variable
   write32b(data->b, *(uint32*)&f);

   res = data->a*2.1;
   // Completed
   return *(uint32*) &res;
}


static int32 MODDECL boolTest(HANDLE* pCPU __attribute__((unused)) , void* pbase)
{
   // Var
   tdef_booltest *data =(tdef_booltest*)pbase;

   // Set access variable
   *data->b = !data->a;

   // Completed
   return data->a;
}



static int32 MODDECL sbTest(HANDLE* pCPU __attribute__((unused)) , void* pbase)
{
   // Var
   int i;
   int16 val;
   tdef_sbtest *pData =(tdef_sbtest*)pbase;

   // Write 7 elements
   for(i=0;i<7;i++){
      val = i+10;
      write16b(&pData->data->elem[i], val);
   }

   // Write number of elements to variable
   write16b(&pData->data->elements, i);
   // Completed
   return i;
}



// Do not use this without a really good reason
static int32 MODDECL getBuf(HANDLE* pCPU __attribute__((unused)) , void* pbase)
{
   // Var
   void* buf;
   tdef_getbuf *data =(tdef_getbuf*)pbase;

   if(data->size<= 0 || data->size > 1000){
      return -1;
   }

   buf = malloc(data->size);
   write32b(data->buf, (int32)buf);

   // Completed
   return 0;
}

static int32 MODDECL freeBuf(HANDLE* pCPU __attribute__((unused)) , void* pbase)
{
   // Var
   void* buf;
   tdef_freebuf *data =(tdef_freebuf*)pbase;

   buf = data->buf;

   free(buf);

   // Completed
   return 0;
}

static int32 MODDECL fbTest(HANDLE* pCPU, void* pbase)
{
   // Var
   char   text[100];
   int16  res;
   uint16 strid;
   tdef_fbtest *data =(tdef_fbtest*)pbase;

   // Calculate
   res = (data->in_a + data->in_b) / 2;

   // Generate output string
   sprintf(text, "(%d + %d) / 2 = %d", data->in_a, data->in_b, res);


   // Update output string
   strid = vplStringUpdate(pCPU, data->str, text);
   data->str = strid;

   // Completed
   return res;
}





/******************************************************************************
 **
 ** FUNCTION TABLE
 **
 *****************************************************************************/
static vplFunctionEntry ftable[] =
{
{ "stringToUpper", stringToUpper },
{ "numberToHex", numberToHex },
{ "sintTest", sintTest },
{ "intTest", intTest },
{ "dintTest", dintTest },
{ "floatTest", floatTest },
{ "boolTest", boolTest },
{ "sbTest", sbTest },
{ "getBuf", getBuf },
{ "freeBuf", freeBuf },
{ "fbTest", fbTest },
{ NULL, NULL } };


/******************************************************************************
 **
 ** EXPORTED FUNCTIONS
 **
 *****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// This function is called by the firmware when 'extModuleLoad' is called.
// Any initialization the module might require is performed here, including
// installing functions and function blocks.
//
// Input:
//   None.
//
// Output:
//   None.
//
// Returns:
//   0                  - Success
//   1                  - Failed
//
/////////////////////////////////////////////////////////////////////////////
int MODDECL moduleInit(void)
{
   // Var
   uint16 rc;

   // Initialize module
   rc = vplInstallFunctions("dt_mod", ftable);
   if (rc)
   {
      // Install failed
      return 1;
   }

   // Completed
   return 0;
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by the firmware to notify the module about an event.
//
// Input:
//   event              - The event ID.
//   parm               - The event parameter.
//
// Output:
//   None.
//
// Returns:
//   None.
//
/////////////////////////////////////////////////////////////////////////////
void MODDECL moduleNotify(int event, int parm)
{
   switch (event)
   {
      case EVENT_HALT:
         break;

      case EVENT_RESET:
         break;

      case EVENT_SHUTDOWN:
         break;

      case EVENT_POWERFAIL:
         if (parm) {
            // Running on battery
         } else {
            // Running on external power
         }
         break;

      case EVENT_POWERSAVE:
         if (parm) {
            // Suspend system
         } else {
            // Resume system
         }
         break;
   }
}


/******************************************************************************
 **
 **    EOF
 **
 *****************************************************************************/
