/***************************** C SOURCE FILE **********************************
 **
 **  Project:    VPL extension module
 **  Filename:   mod_example.c
 **
 ******************************************************************************
 **
 **  This exports a function that converts a number to a hexadecimal string.
 **
 *****************************************************************************/

/******************************************************************************
 **
 ** MODULES USED
 **
 *****************************************************************************/
#include <module.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

/******************************************************************************
 **
 **    TYPEDEFS AND STRUCTURES
 **
 *****************************************************************************/

/*
       ;*** FUNCTION numberToHex ***
       ;*** var
       ;numberToHex                     off=0000 key=791722 sz=02 ty=std ival=0
       ;err                             off=0002 key=791745 sz=02 ty=std ival=0
       ;*** var_input
       ;v                               off=0004 key=749631 sz=04 ty=std ival=0
       ;padding                         off=0008 key=791735 sz=01 ty=std ival=0
       ;*** var_input (access)
 */
typedef struct {
   /*** var */
   int16       retval;
   int16       error;
   /*** var_input */
   int32       v;
   int8        padding;

} ALIGN_ATTR tdef_numbertohex;



/******************************************************************************
 **
 ** LOCAL FUNCTIONS
 **
 *****************************************************************************/



static int32 MODDECL numberToHex(HANDLE* pCPU, void* pbase)
{
   // Var
   char text[255];
   uint16 strid;
   tdef_numbertohex *data =(tdef_numbertohex*)pbase;

   if(data->padding){
      snprintf(text, 255, "%0*lx", data->padding, data->v);
   }else{
      snprintf(text, 255, "%lx", data->v);
   }


   // Allocate string
   strid = vplStringMake(pCPU, text);

   // Completed
   return strid;
}


/******************************************************************************
 **
 ** FUNCTION TABLE
 **
 *****************************************************************************/
static vplFunctionEntry ftable[] =
{
      { "numberToHex", numberToHex },
      { NULL, NULL }
};


/******************************************************************************
 **
 ** EXPORTED FUNCTIONS
 **
 *****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// This function is called by the firmware when 'extModuleLoad' is called.
// Any initialization the module might require is performed here, including
// installing functions and function blocks.
//
// Input:
//   None.
//
// Output:
//   None.
//
// Returns:
//   0                  - Success
//   1                  - Failed
//
/////////////////////////////////////////////////////////////////////////////
int MODDECL moduleInit(void)
{
   // Var
   uint16 rc;

   // Initialize module
   rc = vplInstallFunctions("mod_example", ftable);
   if (rc)
   {
      // Install failed
      return 1;
   }

   // Completed
   return 0;
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by the firmware to notify the module about an event.
//
// Input:
//   event              - The event ID.
//   parm               - The event parameter.
//
// Output:
//   None.
//
// Returns:
//   None.
//
/////////////////////////////////////////////////////////////////////////////
void MODDECL moduleNotify(int event, int parm)
{
   switch (event)
   {
      case EVENT_HALT:
         break;

      case EVENT_RESET:
         break;

      case EVENT_SHUTDOWN:
         break;

      case EVENT_POWERFAIL:
         if (parm) {
            // Running on battery
         } else {
            // Running on external power
         }
         break;

      case EVENT_POWERSAVE:
         if (parm) {
            // Suspend system
         } else {
            // Resume system
         }
         break;
   }
}


/******************************************************************************
 **
 **    EOF
 **
 *****************************************************************************/
