/***************************** C SOURCE FILE **********************************
 **
 **  Project:    VPL extension module
 **  Filename:   helloworld.c
 **
 ******************************************************************************
 **
 **  This exports a function which returns a string with 'Hello World'
 **
 *****************************************************************************/

/******************************************************************************
 **
 ** MODULES USED
 **
 *****************************************************************************/
#include <module.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "mxml/mxml.h"

/******************************************************************************
 **
 **    DEFINITIONS AND MACROS
 **
 *****************************************************************************/
#define WAV_HEADER_LEN 44

/******************************************************************************
 **
 **    TYPEDEFS AND STRUCTURES
 **
 *****************************************************************************/

/*
 ;*** FUNCTION AudioTest ***
 ;*** var
 ;AudioTest                       off=0000 key=93154 sz=02 ty=std ival=0
 ;rc                              off=0002 key=73829 sz=02 ty=std ival=0
 ;*** var_input
 ;id                              off=0004 key=93162 sz=02 ty=std ival=1
 ;playback                        off=0006 key=93168 sz=01 ty=std ival=1
 ;capture                         off=0007 key=93176 sz=01 ty=std ival=1
 ;*** var_input (access)
 */
typedef struct
{
   /*** var */int16 retval;
   int16 error;
   /*** var_input */int16 id;
   int8 playback;
   int8 capture;

}ALIGN_ATTR tdef_audiotest;

/******************************************************************************
 **
 ** GLOBAL VARIABLES
 **
 *****************************************************************************/

/******************************************************************************
 **
 ** LOCAL FUNCTIONS
 **
 *****************************************************************************/
#include <unistd.h>

static int32 MODDECL xml_test(__attribute__((unused))HANDLE* pCPU,
                              __attribute__((unused)) void* pbase)
{
   // Var
   int rc = 0;
   FILE *fp = NULL;
   mxml_node_t *tree;
   char file[200];
   mxml_node_t *value;
   const char* name = NULL;

   if (vplFsMapFile(file, 200, "B:\\TEST.XML"))
   {
      vplDebug("Failed to parse path:");
      return -1;
   }

   fp = fopen(file, "r");
   if (!fp)
   {
      vplDebug("Failed to open file");
      return -2;
   }
   tree = mxmlLoadFile(NULL, fp, MXML_NO_CALLBACK);

   fclose(fp);
   if (!tree)
   {
      vplDebug("Failed to parse file");
      return -3;
   }

   value = mxmlFindPath(tree, "node");
   if (value)
   {
      name = mxmlElementGetAttr(value, "name");
      if (name)
      {
         const char* found_msg = "Name: ";
         char* msg = malloc(strlen(name)+strlen(found_msg)+1);

         strcpy(msg, found_msg);
         strcat(msg, name);

         vplDebug(msg);
      }
      else
      {
         vplDebug("Failed to find name");
      }
   }
   else
   {
      vplDebug("Failed to finde node");
   }

   mxmlDelete(tree);

   // Completed
   return rc;
}

/******************************************************************************
 **
 ** FUNCTION TABLE
 **
 *****************************************************************************/
static vplFunctionEntry ftable[] =
{
{ "xml_test", xml_test },
{ NULL, NULL } };

/******************************************************************************
 **
 ** EXPORTED FUNCTIONS
 **
 *****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
// This function is called by the firmware when 'extModuleLoad' is called.
// Any initialization the module might require is performed here, including
// installing functions and function blocks.
//
// Input:
//   None.
//
// Output:
//   None.
//
// Returns:
//   0                  - Success
//   1                  - Failed
//
/////////////////////////////////////////////////////////////////////////////
int MODDECL moduleInit(void)
{
   // Var
   uint16 rc;

   // Initialize module
   rc = vplInstallFunctions("xml_test", ftable);
   if (rc)
   {
      // Install failed
      return 1;
   }

   // Completed
   return 0;
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by the firmware to notify the module about an event.
//
// Input:
//   event              - The event ID.
//   parm               - The event parameter.
//
// Output:
//   None.
//
// Returns:
//   None.
//
/////////////////////////////////////////////////////////////////////////////
void MODDECL moduleNotify(int event, int parm)
{
   switch (event)
   {
   case EVENT_HALT:
      break;

   case EVENT_RESET:
      break;

   case EVENT_SHUTDOWN:
      break;

   case EVENT_POWERFAIL:
      if (parm)
      {
         // Running on battery
      }
      else
      {
         // Running on external power
      }
      break;

   case EVENT_POWERSAVE:
      if (parm)
      {
         // Suspend system
      }
      else
      {
         // Resume system
      }
      break;
   }
}

/******************************************************************************
 **
 **    EOF
 **
 *****************************************************************************/
